use strict;
use warnings;
package Test::Routine::Manual::Demo;
BEGIN {
  $Test::Routine::Manual::Demo::VERSION = '0.005';
}
# ABSTRACT: a walkthrough, in code, of Test::Routine


1;

__END__
=pod

=head1 NAME

Test::Routine::Manual::Demo - a walkthrough, in code, of Test::Routine

=head1 VERSION

version 0.005

=head1 The Demo



=head1 AUTHOR

Ricardo Signes <rjbs@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2010 by Ricardo Signes.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

